// functions over channels: a server which receives a function,
// execute it and return the obtained value

package main

import "fmt"

// the client send a request, including a function to execute, the
// argument and the channel where the answer should be returned
type Request struct { 
	Fun       func (int) int   
	Arg       int
	ReplyChan chan int
}


// serve: a function which satisfy a request
func serve(req *Request) {
	req.ReplyChan <- req.Fun(req.Arg)
}


// server, parametrised by a port for receiving
func server(service <-chan *Request) {
	var req *Request
	for {
		req = <-service  // requests arrive here 
		go serve(req)    // don't wait for op
	}
}


// function for starting the server: it returns the port for sending requests
func startServer() chan<- *Request {
	service := make(chan *Request) 
	go server(service) 
	return service
}


func main() {
	var localVar int;

	// create a server instance
	serverChan := startServer()

	// note: closures are passed!
	localVar=2
	req1 := &Request{
		func (a int) int { return localVar * a }, 
		3, 
		make(chan int)} 

	req2 := &Request{
		func (a int) int { return a*a }, 
		6, 
		make(chan int)}

	serverChan <- req1 
	serverChan <- req2
	
	fmt.Printf("Request 2: %d\n", 
		<-req2.ReplyChan)

	fmt.Printf("Request 1: %d\n", 
		<-req1.ReplyChan)

}	
