/* Computes Mandelbrot set: sequential or one goroutine per point */

/*
 * to see command line parameters enter: -flag
 * to enter a command line parameter enter: -<param>=<entry>
 */


package main

//import "runtime"
//import "fmt"
//import "math"
import "image"
import "image/color"
import "flag"


import "lcd.it/mandelpkg"


func computePoint(i, j, imgW, imgH, maxIters int, x0, y0, offx, offy float64, colorIn, colorOut color.RGBA, c chan *mandelpkg.ColoredPoint) {
	point := mandelpkg.NewColoredPoint(i,j,color.RGBA{0, 0, 0, 0})
	mandelpkg.ElaboratePoint(&point, imgW, imgH, maxIters, x0, y0, offx, offy, colorIn, colorOut)
	c <- &point
}


func main() {
	
	// colors (inner and outer, a smooth transition is done)
	colorIn  := color.RGBA{0,0, 0, 0xFF}
	colorOut := color.RGBA{15, 0, 0, 0xFF}

	
	// image parameters
	var imagePointSize int
	var x0 float64
	var y0 float64
	var offx float64
	var offy float64
	var maxIters int
	var fOut string
	
	// optional input from the command line with default values
	flag.IntVar(&imagePointSize, "imagePointSize", 800, "X-size of the image in points")
	flag.Float64Var(&x0, "x0", -2.5, "x origin for the image")
	flag.Float64Var(&y0, "y0", -1, "y origin for the image")
	flag.Float64Var(&offx, "offx", 3.5, "X-offset")
	flag.Float64Var(&offy, "offy", 2, "Y-offset")
	flag.IntVar(&maxIters, "maxIters", 10000, "Bound for iterations")
	flag.StringVar(&fOut, "f", "mandelbrot", "File name for the image (without jpg suffix)")


	
	//fmt.Println(runtime.GOMAXPROCS(-1))

	imgW := imagePointSize
	imgH := int(float64(imagePointSize)*offy/offx)
	// place inputs in variables
	flag.Parse()
	
	// create the image
	var image *image.RGBA = image.NewRGBA(image.Rectangle{image.Point{0,0}, image.Point{imgW, imgH}})


	// channel where colored points are inserted once produced
	c := make(chan *mandelpkg.ColoredPoint, imgW*imgH)

	// for each pixel, elaborate the corresponding color
	for i := 0; i < imgW; i++ {
		for j := 0; j < imgH; j++ {
			go computePoint(i, j, imgW, imgH, maxIters, x0, y0, offx, offy, colorIn, colorOut, c)
		}
	}
	
	for i:=0; i < imgW * imgH; i++ {
		point := <-c
		image.Set(point.GetComponents())
	}

	// write to the .jpg
	mandelpkg.WriteImage(image, fOut)
}
