/* Computes Mandelbrot set: solution with a producers, generating the canidate points and a fixed number of consumers (workers), computing the color */

/*
 * to see command line parameters enter: -flag
 * to enter a command line parameter enter: -<param>=<entry>
 */


package main


//import "math"
import "image/color"
import "flag"
import "image"
//import "time"

import "lcd.it/mandelpkg"


// get a point, colores it and send it to output
func computePoint(imgW, imgH, maxIters int, x0, y0, offx, offy float64, colorIn, colorOut color.RGBA, c, out chan *mandelpkg.ColoredPoint) {
	for {
		point := <- c
		mandelpkg.ElaboratePoint(point, imgW, imgH, maxIters, x0, y0, offx, offy, colorIn, colorOut)
		out<- point
	}
}


func pointProducer(imgW, imgH int, c chan *mandelpkg.ColoredPoint) {
	for i := 0; i < imgW; i++ {
		for j := 0; j < imgH; j++ {
			point := mandelpkg.NewColoredPoint(i,j,color.RGBA{0, 0, 0, 0})
			c <- &point
		}
	}
}

func main() {

	// colors (inner and outer, a smooth transition is done)
	colorIn  := color.RGBA{0,0, 0, 0xFF}
	colorOut := color.RGBA{15, 0, 0, 0xFF}

	// image parameters
	var imagePointSize int
	var x0 float64
	var y0 float64
	var offx float64
	var offy float64
	var maxIters int
	var fOut string

	// number of workers
	var numWorkers int
	
	// optional input from the command line with default values
	flag.IntVar(&imagePointSize, "imagePointSize", 800, "X-size of the image in points")
	flag.Float64Var(&x0, "x0", -2.5, "x origin for the image")
	flag.Float64Var(&y0, "y0", -1, "y origin for the image")
	flag.Float64Var(&offx, "offx", 3.5, "X-offset")
	flag.Float64Var(&offy, "offy", 2, "Y-offset")
	flag.IntVar(&maxIters, "maxIters", 10000, "Bound for iterations")
	flag.StringVar(&fOut, "f", "mandelbrot", "File name for the image (without png suffix)")
	flag.IntVar(&numWorkers, "numWorkers", 8, "Number of workers")


	imgW := imagePointSize
	imgH := (int)(float64(imagePointSize)*offy/offx)
	// place inputs in variables
	flag.Parse()
		
	//img1 := color.RGBA{40, 0, 0, 0xFF} // background color
	//img2 := color.RGBA{255, 0, 0, 0xFF}
	
	// create the image
	var mandelImg *image.RGBA = image.NewRGBA(image.Rectangle{image.Point{0,0}, image.Point{imgW, imgH}})

	// channel for communication producer->consumers
	c := make(chan *mandelpkg.ColoredPoint, imgW*imgH)
	// channel for outputs
	out := make(chan *mandelpkg.ColoredPoint, imgW*imgH)


	for i := 0; i < numWorkers; i++ {
		go computePoint(imgW, imgH, maxIters, x0, y0, offx, offy, colorIn, colorOut, c, out)
	}

	// fill the channels with uncolored points
	go pointProducer(imgW, imgH, c)
	
	
	for i:=0; i < imgW * imgH; i++ {
		point := <-out
		mandelImg.Set(point.GetComponents())
	}

	// write to the .png
	mandelpkg.WriteImage(mandelImg, fOut)
}
