/* ufficio 
 Off = (CTM | CS) \ L
 CTM = pay. (tau. 'tea.CTM + tau. 'coffee.CTM) 
 
 CS  = publish.'pay.coffee.CTM

 oppure

 CS  = publish.'pay.(tau. coffee.CTM + tau. tea.CTM)
*/

package main

import ( 
	"fmt" 
	"math/rand"
	"time"
	)


// Coffee&tea machine
type CTM struct {
	Pay    chan int     //  get money
	Coffee chan int     //  coffee or tea request
	Tea    chan int
}


// CTM behaviour
func (self *CTM) Run () {
	var choice string
	for {
		// get the money
		<- self.Pay   
		fmt.Printf("CTM: Got a coin\n")
		
		// internal choice
		if (rand.Intn(2) == 0) {
			<- self.Tea
			choice = "Tea"
		} else {
			<- self.Coffee
			choice = "Coffee"
		}
		
		fmt.Printf("CTM: %s chosen\n", choice)	
	}
}


// CS (publishing channel omitted for the sake of simplicity)

func CS (machine *CTM) {
	// kind of beverage chosen
	var beverage string

	for {
		// publish
		fmt.Printf("CS: publish!\n")
		
		// pay
		machine.Pay <- 1
		fmt.Printf("CS: pay\n")

		//  choose coffee (or, randomly, tea if not commented)
		if (rand.Intn(2) == 0) {
			//select {
			
			//case
			machine.Coffee <- 1
			fmt.Printf("CS asked for coffee\n")
			beverage = "coffee"
			} else {
			/* comment this for the CS willing only to have coffee */
			// case
			machine.Tea <- 1
			fmt.Printf("CS: asked for tea\n")
			beverage = "tea"
			}
		
		fmt.Printf("CS: got %s\n", beverage)
		
	}
}
	
// main
func main () {
	var machine CTM

	// seeding
	rand.Seed(time.Now().UnixNano())
	
	
	machine = CTM{
		make(chan int),
		make(chan int),
		make(chan int)}


	go machine.Run()

	CS(&machine)
		
}
