/*
 versione elaborata, con valori, timeout ecc.  Il CS in cambio di
 una pubblicazione riceve del denaro random, con il quale prova a
 comprare qualcosa (usando tutto il denaro che ha). Prima di scegliere
 la bevanda aspetta un tempo random, che potrebbe portare la macchina
 a restituire il denaro (timeout). Se il denaro e' sufficiente ottiene
 la bevanda ed il resto, altrimenti, se il denaro non basta, comunque
 la macchina restituisce il denaro.  */

package main

import ( 
	"fmt" 
	"time"
	"math/rand"
	)



// Coffee&tea machine
type CTM struct {
	Pay    chan int     //  get money
	Coffee chan int     //  asking coffee or tea
	Tea    chan int
	Drink  chan string  // channel for providing the drink
	Change chan int     // change, if any
	TOut   chan int     // timeout signal for the user
}


// CTM behaviour
func (self *CTM) Run () {

	// map of prices
	var Price = make(map[string]int)
	Price["Tea"] = 3
	Price["Coffee"] = 4
	
	// user choice (coffee or tea)
	var choice string
	// money inserted
	var money  int

	for {
		// get the money
		money = <- self.Pay   
		fmt.Printf("CTM: Getting %d cents\n", money)
		
		// wait for the choice (with timeout)
		select {
		case <- self.Tea: 
			choice = "Tea"
			fmt.Println("CTM:Tea")

		case <- self.Coffee:
			choice = "Coffee"
			fmt.Println("CTM:Coffee")

		case <- time.After(3 * time.Second): // time.After(3*1e9)
		  	fmt.Println("CTM: Timed out after 3 seconds")
		   	choice = "nothing"
			// if timed out, tell the user
			self.TOut <- 1
		}


		fmt.Printf("CTM: %s chosen\n", choice)

		
		if choice != "nothing" {
			// if some choice has been taken

			// check if money is enough
			if Price[choice] <= money {
				// provide the beverage
				self.Drink <- choice
				// take the money
				// the remaining money is the change
				money = money - Price[choice]
			} else {
				fmt.Printf("CTM: %d cents not enough money for a %s\n", 
					money, choice)
			}
		}
		
		// give back the change 
		self.Change <- money
			
	}
		
	
}


// CS (publishing channel omitted for the sake of simplicity)

func CS (machine *CTM) {
	var money int = 0
	var beverage string

	for {
		// publish and get some random money
		gain := rand.Intn(5)
		fmt.Printf("CS publishes and gains %d cents\n", gain)
		money = money + gain

		// pay
		fmt.Printf("CS pay %d\n", money)
		machine.Pay <- money


		// wait random time betweem 0 and 5 seconds (could timeout)
		time.Sleep(time.Duration(rand.Intn(5))*time.Second)

		// randomly choose coffee or tea (or nothing, if timed-out)
		select {
		case machine.Tea <- 1:
			fmt.Printf("CS asking for tea\n")
			
		case machine.Coffee <- 1:
			fmt.Printf("CS asking for coffee\n")
			
		case <- machine.TOut:
		    	fmt.Printf("CS:timed out\n")
		}

		
		// could get the beverage or only the money back
		// avoid duplicating input on  money?
		select {
		case beverage = <-machine.Drink:
			fmt.Printf("CS got %s\n", beverage)
			money = <- machine.Change
			
		case money = <- machine.Change:
			fmt.Printf("CS got no drink\n")
		}

		fmt.Printf("CS change back %d \n", money)
	}

}

	
// main
func main () {
	
	// seeding 
 	rand.Seed(time.Now().UnixNano())


	var machine CTM
	
	machine = CTM{
		make(chan int),
		make(chan int),
		make(chan int),
		make(chan string),
		make(chan int),
		make(chan int)}


	go machine.Run()

	CS(&machine)

}
