// given an array, apply a given function to all elements


package main

import (
	"fmt"
	"time"
	"sync"
	)

/* REQUEST *****************************************************/

type Operator func (int) int

type Request struct {
	Op        Operator  // operator to be applied
	InputChan chan int  // inputs elements to be processed 
	ReplyChan chan int  // returns processed elements
}

/* SERVER *****************************************************/

// parametrised by an input port for receiving
func server(service <-chan *Request) {
	for {
		req := <-service    // requests arrive here 
		go serve(req)       // don't wait for op
	}
}

// function serving the request: it inputs all elements from
// InputChan, create a process for each input, collect the results,
// send them to the ReplyChannel and close it
func serve(req *Request) {
	// use WaitGroup
	var wg sync.WaitGroup
	for val := range req.InputChan {
		//concurrent or non-concurrent, this is the problem
		wg.Add(1)
		//go slave(req.Op, val, req.ReplyChan, &wg)
		slave(req.Op, val, req.ReplyChan, &wg)
		
	}
	wg.Wait()
	close(req.ReplyChan)
}



// slave function for processing a single element
func slave(op Operator, val int, reply chan<- int, wg *sync.WaitGroup) {
	defer wg.Done()
	reply <- op(val)
	
}
	


// function for starting the server: returns a port for sending requests
func startServer() chan<- *Request {
	service := make(chan *Request) 
	go server(service) 
	return service
}


func fact (n int) int {
	if (n == 0) {
		return 1
	} else {
		time.Sleep(time.Duration(10)*time.Millisecond)
		return n * fact(n-1)
	}
}

func main() {
	// create a server instance
	serverChan := startServer()
	
	// client side
	req := &Request{
		fact,
		make(chan int, 100),
		make(chan int, 100)} 

	// elements to be processed [0, 1, 2, ..., NProc]
	const Max = 45
	
	// send the requests
	serverChan <- req

	for val := 0; val < Max; val++ {
		req.InputChan <- val
	}
	close(req.InputChan)
	
	// retrieve the results ranging on the return channel
	sum := 0
	for v := range req.ReplyChan {
		sum += v
	}

	fmt.Printf("Request out: %d\n", sum)
}
