package main

import ( 
	"fmt" 
 	"time"
	"math/rand"
	)

// empty structs used for signalling on channels
type Signal struct{}
var sigVal = struct{}{}


// Fork type
type Fork struct {
	name string      //  fork name
	free bool        //  is it free?

	// (public) channels
	Take chan Signal     //  request of taking
	Leave chan Signal     //  release
}

// Fork behaviour
func (self *Fork) Run () {
	
	for {
		if self.free {
			<- self.Take
			self.free = false
			fmt.Printf("Fork %s taken\n", self.name) 
		} else {
			<- self.Leave
			self.free = true
 			fmt.Printf("Fork %s released\n", self.name)      
		}
	}
	
}

// Philosopher behaviour

func Phil (id int, left *Fork, right *Fork) {
	for {
		fmt.Printf("Philosopher %d is thinking ...\n", id)
		/* sleep for at most 2 seconds */
		time.Sleep(time.Duration(rand.Int63n(2*1e9)))
		/* take left fork */
		left.Take <- sigVal
		/* sleep for at most 2 seconds */
		time.Sleep(time.Duration(rand.Int63n(2*1e9)))
		/* take right fork */
		right.Take <- sigVal
		fmt.Printf("Philosopher %d is eating ...\n", id)
		left.Leave <- sigVal
		right.Leave <- sigVal
	}
}



// Number of philosophers
const NPhil = 5

// main
func main () {
	var forks [NPhil]Fork

	// seeding
	rand.Seed(time.Now().UnixNano())
	
	for i:=0; i<NPhil; i++ {

		// create the ith-fork
		forks[i] = Fork { 
			fmt.Sprintf("F%d",i), 
			true, 
			make(chan Signal), 
			make(chan Signal)}

		go forks[i].Run()
	}
	
	for i:=0; i<NPhil-1; i++ {
		go Phil(i, &forks[i], &forks[(i+1)%NPhil])
	}
	
	// left-handed last philospher: no deadlock!
	// Phil(NPhil-1, &forks[0], &forks[NPhil-1])

	// right-handed last philospher: deadlock!
	Phil(NPhil-1, &forks[NPhil-1], &forks[0])
}

