/* shows the major flexibility of the channel based solution:
 the object decide how to manage, possibly merge the requests */

package main

import (
	"fmt"
	"time"
	)

// a structure
type MyObj struct {	
	X        int        //  a value
	DoubleX  int        //  its double
	IncReq   chan bool  // inc requests
	DecReq   chan bool  // dec requests
}


// increment no longer public
func (self *MyObj) inc () {
	self.X++
	self.DoubleX=self.X
	self.DoubleX=self.DoubleX*2
	fmt.Println("object:", self)
}

func (self *MyObj) dec () {
	self.X--
	self.DoubleX=self.X
	self.DoubleX=self.DoubleX*2
	fmt.Println("object:", self)
}

// expose inc as a "service"
func New (X int) *MyObj {
	
	obj:=MyObj{X, 2*X,make(chan bool), make(chan bool)}
	
	go func () {
		for {
			select {
			case <-obj.IncReq:
				time.Sleep(time.Duration(1 * time.Millisecond))

				select {
				case <- obj.DecReq:
					fmt.Println("Inc and Dec, doing nothing")
				default:
					obj.inc()
				}
				
			case <-obj.DecReq:
				time.Sleep(time.Duration(1 * time.Millisecond))
				
				select {
				case <- obj.IncReq:
					fmt.Println("Inc and Dec, doing nothing")
				default:
					obj.dec()
				}
				
			}
		}
	}()
	return &obj
}


func child (comm chan *MyObj, end chan bool) {
	val := <-comm
	val.IncReq <- true
	end <- true
}

func main() {
	obj := New(2)
	fmt.Println("object:", obj)

	// channel for sending objects
	c := make(chan *MyObj)
	// channel for waiting the end of the child (not paradigmatic
	// ... see later examples for better solutions)
	wait := make(chan bool)


	// adhere to channel-based communication, but still ... I can
	// exchange reference values (pointers) ... (and this is not
	// considered bad practice)	
	go child(c, wait)

	// send ref to obj on the channel (received by the child)
	c <- obj

	// act on the object
	obj.DecReq <- true
	
	// wait for child termination
	<-wait

	// just to avoid errors on unused packages
	time.Sleep(time.Duration(1 * time.Millisecond))
}
