package main

import (
	"fmt"
	"time"
	)

// a structure
type MyObj struct {	
	X        int        //  a value
	DoubleX  int        //  its double
	IncReq   chan bool  // inc requests
}


// increment no longer public
func (self *MyObj) inc () {
	self.X++
	self.DoubleX=2*self.X
	fmt.Println("object:", self)
}

// expose inc as a "service"
func New (X int) *MyObj {
	IncReq:=make(chan bool)
	obj:=MyObj{X, 2*X,IncReq}
	
	go func () {
		for {
			<-IncReq
			obj.inc()
		}
	}()
	return &obj
}


func child (comm chan *MyObj, end chan bool) {
	val := <-comm
	val.IncReq <- true
	end <- true
}

func main() {
	obj := New(2)
	fmt.Println("object", obj)

	// channel for sending objects
	c := make(chan *MyObj)
	// channel for waiting the end of the child (not paradigmatic
	// ... see later examples for better solutions)
	wait := make(chan bool)


	// adhere to channel-based communication, but still ... I can
	// exchange reference values (pointers) ... (and this is not
	// considered bad practice)	
	go child(c, wait)

	// send ref to obj on the channel (received by the child)
	c <- obj

	// act on the object
	obj.IncReq <- true
	
	// wait for child termination
	<-wait

	// just to avoid errors on unused packages
	time.Sleep(time.Duration(1 * time.Millisecond))
}
