package main

import (
	"fmt"
	"time"
	"sync"
	)

// a structure
type MyObj struct {	
	X        int     //  a value
	DoubleX  int     //  its double
	L        sync.Mutex // a lock	
}

func New (X int) *MyObj {
	return &MyObj{X, 2*X, *new(sync.Mutex)}
}

func (self *MyObj) Inc() {
	self.L.Lock()            // take the lock
	defer self.L.Unlock()

	self.X++
	
	// double X value in DoubleX: clearly, not the best way to do
	// it, but fits our aims.
	self.DoubleX = self.X
	// use sleep to induce some scheduling
	time.Sleep(time.Duration(1*time.Millisecond))
	self.DoubleX = 2 * self.DoubleX
	fmt.Println("object:", self)
}

func child (comm chan *MyObj, end chan bool) {
	val := <-comm
	val.Inc()
	end <- true
}

func main() {
	obj := New(2)
	fmt.Println("object:", obj)

	// channel for sending objects
	c := make(chan *MyObj)
	// channel for waiting the end of the child (not paradigmatic
	// ... see later examples for better solutions)
	wait := make(chan bool)


	// adhere to channel-based communication, but still ... I can
	// exchange reference values (pointers) ... (and this is not
	// considered bad practice)	
	go child(c, wait)

	// send ref to obj on the channel (received by the child)
	c <- obj

	// act on the object
	obj.Inc()
	
	// wait for child termination
	<-wait

	// just to avoid errors on unused packages
	time.Sleep(time.Duration(1 * time.Millisecond))
}
